﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Windows.Devices.Enumeration;
using Windows.Devices.Spi;


namespace SPIADC
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        private SpiDevice myADC;
        private const byte SPI_CHIP_SELECT_LINE = 0;
        private byte[] Channel0 = { 0x68, 0x00 };
        private byte[] Channel1 = { 0x78, 0x00 };
        private byte[] DiffCh0PCh1N = { 0x48, 0x0 };
        private byte[] DiffCh0NCh1P = { 0x58, 0x0 };

#pragma warning disable CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        public MainWindow()
#pragma warning restore CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        {
            InitializeComponent();
            SPISetup();
        }

        private async void SPISetup()
        {

            try
            {
                SpiConnectionSettings spiSettings = new SpiConnectionSettings(SPI_CHIP_SELECT_LINE);
                spiSettings.ClockFrequency = 8000000;
                spiSettings.Mode = SpiMode.Mode0;

                string aqs = SpiDevice.GetDeviceSelector("SPI2");
                DeviceInformationCollection spiDeviceInfo = await DeviceInformation.FindAllAsync(aqs);
                myADC = await SpiDevice.FromIdAsync(spiDeviceInfo[0].Id, spiSettings);

                if (myADC != null)
                {
                    txtStatus.Text = "ADC is ready";
                }
                else
                {
                    txtStatus.Text = "ADC is NOT ready";
                }
            }
            catch (Exception ex)
            {
                txtStatus.Text = ex.Message;
            }
        }

        private void btnReadChannel0_Click(object sender, RoutedEventArgs e)
        {
            byte[] adcCh0Out = new byte[2];
            myADC.TransferFullDuplex(Channel0, adcCh0Out);
            var adcCh0OutInt = (adcCh0Out[0] << 8 | adcCh0Out[1]) & 0x3FF;


            if (rb3v.IsChecked == true)
            {
                txtChannel0Result.Text = Math.Round((double)adcCh0OutInt / 1024 * 3.3, 2).ToString();

            }
            if (rb5v.IsChecked == true)
            {
                txtChannel0Result.Text = Math.Round((double)adcCh0OutInt / 1024 * 5, 2).ToString();
            }

        }

        private void btnReadChannel1_Click(object sender, RoutedEventArgs e)
        {
            byte[] adcCh1Out = new byte[2];
            myADC.TransferFullDuplex(Channel1, adcCh1Out);

            var adcCh1OutInt = (adcCh1Out[0] << 8 | adcCh1Out[1]) & 0x3ff;


            if (rb3v.IsChecked == true)
            {
                txtChannel1Result.Text = Math.Round((double)adcCh1OutInt / 1024 * 3.3, 2).ToString();

            }
            if (rb5v.IsChecked == true)
            {
                txtChannel1Result.Text = Math.Round((double)adcCh1OutInt / 1024 * 5, 2).ToString();
            }

        }

        private void btnCh0PCh1N_Click(object sender, RoutedEventArgs e)
        {
            byte[] adcDiffOut = new byte[2];
            myADC.TransferFullDuplex(DiffCh0PCh1N, adcDiffOut);

            var adcDiffOutInt = (adcDiffOut[0] << 8 | adcDiffOut[1]) & 0x3ff;

            if (adcDiffOutInt > 0) //if IN+ is equal to or bloew IN- the result is 0x00 so we test for 0
            {
                if (rb3v.IsChecked == true)
                {
                    txtChannelDiff.Text = Math.Round((double)adcDiffOutInt / 1024 * 3.3, 2).ToString();
                }
                if (rb5v.IsChecked == true)
                {
                    txtChannelDiff.Text = Math.Round((double)adcDiffOutInt / 1024 * 5, 2).ToString();
                }
            }
            else
            {
                myADC.TransferFullDuplex(DiffCh0NCh1P, adcDiffOut); //Flip the differential input to test for negative
                adcDiffOutInt = (adcDiffOut[0] << 8 | adcDiffOut[1]) & 0x3ff;
                if (rb3v.IsChecked == true)
                {
                    txtChannelDiff.Text = (-Math.Round((double)adcDiffOutInt / 1024 * 3.3, 2)).ToString();
                }
                if (rb5v.IsChecked == true)
                {
                    txtChannelDiff.Text = (-Math.Round((double)adcDiffOutInt / 1024 * 5, 2)).ToString();
                }
            }
        }
    }
}
