using System;
using Windows.Devices.Enumeration;
using Windows.Devices.I2c;


namespace Annabooks.TMP102
{
    public class TMP102
    {

        private I2cDevice sensorTMP102;
        private const byte TempRegisterAddr = 0x00;
        private const byte ConfigRegisterAddr = 0x01;
        private const byte TlowRegisterAddr = 0x02;
        private const byte ThighRegisterAddr = 0x03;

        private bool tempReady = false;
        public string connectionError = "";

#pragma warning disable CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        public TMP102(string I2Cport, bool I2CBusShared, byte addr)
#pragma warning restore CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        {
            switch (addr)
            {
                case 0x48:
                    I2CInitialize(I2Cport, I2CBusShared, 0x48);
                    break;
                case 0x49:
                    I2CInitialize(I2Cport, I2CBusShared, 0x49);
                    break;
                case 0x4A:
                    I2CInitialize(I2Cport, I2CBusShared, 0x4A);
                    break;
                case 0x4B:
                    I2CInitialize(I2Cport, I2CBusShared, 0x4B);
                    break;
                default:
                    tempReady = false;
                    break;
            }
        }
        private async void I2CInitialize(string I2Cport, bool I2CBusShared, byte addr)
        {
            try
            {
                I2cConnectionSettings i2cSettings = new I2cConnectionSettings(addr);
                i2cSettings.BusSpeed = I2cBusSpeed.FastMode;
                if (I2CBusShared)//True
                {
                    i2cSettings.SharingMode = I2cSharingMode.Shared;
                }
                else//false
                {
                    i2cSettings.SharingMode = I2cSharingMode.Exclusive;
                }
                string aqs = I2cDevice.GetDeviceSelector(I2Cport);
                DeviceInformationCollection i2cDevInfo = await DeviceInformation.FindAllAsync(aqs);
                sensorTMP102 = await I2cDevice.FromIdAsync(i2cDevInfo[0].Id, i2cSettings);
            }
            catch (Exception ex)
            {
                connectionError = ex.Message;
            }

            if (sensorTMP102 != null)
            {
                tempReady = true;
            }
        }

        public double ReadTempC()
        {
            byte[] Readbuf = new byte[2];
            byte[] RegAddrBuf = new byte[] { TempRegisterAddr };
            sensorTMP102.WriteRead(RegAddrBuf, Readbuf);

            byte tempMSB = Readbuf[0];
            var tempMSBs = tempMSB << 4;
            byte tempLSB = Readbuf[1];
            var tempLSBs = tempLSB >> 4;
            var tempFinal = tempMSBs + tempLSBs;
            if ((tempFinal & 0x800) == 0x800) //detect if negative
            {
                tempFinal = (tempFinal ^ 0xFFF) + 1; //twos compliment
                tempFinal = -tempFinal;
            }
            double tempC = Convert.ToDouble(tempFinal) * 0.0625;
            return tempC;
        }

        public double ReadTempF()
        {
            byte[] Readbuf = new byte[2];
            byte[] RegAddrBuf = new byte[] { TempRegisterAddr };
            sensorTMP102.WriteRead(RegAddrBuf, Readbuf);

            byte tempMSB = Readbuf[0];
            var tempMSBs = tempMSB << 4;
            byte tempLSB = Readbuf[1];
            var tempLSBs = tempLSB >> 4;
            var tempFinal = tempMSBs + tempLSBs;
            if ((tempFinal & 0x800) == 0x800) //detect if negative
            {
                tempFinal ^= 0xFFF; //twos compliment
                tempFinal = -tempFinal;
            }
            double tempC = Convert.ToDouble(tempFinal) * 0.0625;
            double tempF = (tempC * 1.8) + 32.0;
            return tempF;

        }

        public bool TMP102Status()
        {
            return tempReady;
        }

    }
}
